/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

// @(#) $Id: NotificationList.java 2093 2007-11-28 14:23:36Z s3460 $
package at.spardat.enterprise.exc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;

/**
 * A <tt>NotificationList</tt> is a {@link Notification} which acts as container
 * for other notifications. Therefore, this class enriches <tt>Notification</tt>
 * with the capability to add other notifications to this, see {@link #add(INotification)} and
 * to iterate over the contained <tt>Notifications</tt>, see {@link #iterator}. <p>
 * 
 * Since the contained <tt>INotification</tt>s may also be <tt>NotificationList</tt>s, 
 * a <tt>NotificationList</tt> may hold a tree of <tt>INotifications</tt>. <p>
 * 
 * The type of the notification may be explicitely set via {@link #setType}. If no
 * type is set, the type is derived from the most severe type of the childs.
 * The order of type severity is (from most severe to least severe): T_ERROR, 
 * T_WARNING, T_INFORMATION, T_QUESTION.  
 * 
 * @author YSD, 22.05.2003 09:27:54
 */
public class NotificationList extends Notification {
    
    /**
     * The notifications aggregated by this.
     */
    private ArrayList           notifications_ = new ArrayList();
    
    /**
     * Defines that the type should be derived from the contained Notifcations
     * in this. 
     */
    private boolean             autoType_ = true;
    
    
    
    /**
     * Constructs and sets the <tt>message</tt> of this from a two
     * parameter <tt>java.text.MessageFormat</tt> compatible string.
     * 
     * @param messageFmt format string as defined in <tt>java.text.MessageFormat</tt>.
     * @param param1 first message parameter
     * @param param2 second message parameter
     */
    public NotificationList (String messageFmt, Object param1, Object param2) {
        this (messageFmt, null, new Object[]{param1,param2});
    }
    
    /**
     * Constructs and sets the <tt>message</tt> of this from a one
     * parameter <tt>java.text.MessageFormat</tt> compatible string.
     * 
     * @param messageFmt format string as defined in <tt>java.text.MessageFormat</tt>.
     * @param param1 message parameter
     */
    public NotificationList (String messageFmt, Object param1) {
        this (messageFmt, null, new Object[]{param1});
    }
    
    /**
     * Constructs and sets the message to the provided value.
     * 
     * @param message the message to set.
     */
    public NotificationList (String message) {
        this (message, null, null);
    }
    
    /**
     * Constructs this with default values. Use one of the <tt>setMessage</tt>-methods
     * to set the message for this compound notification.
     */
    public NotificationList () {
        this ("");
    }
    
    /**
     * Constructs and sets the <tt>message</tt> from a format string as
     * defined in <tt>java.text.MessageFormat</tt>. The required parameters
     * must be contained in the <tt>params</tt> array.
     * 
     * @param message    format string as defined in <tt>java.text.MessageFormat</tt> or
     *                    plain text, depending on <tt>params</tt>.
     * @param l          a <tt>java.util.Locale</tt> to format locale 
     *                    dependent data types or <tt>null</tt> if the
     *                    <tt>params</tt> do not contain local specifics.
     * @param params the parameters of the message. If <tt>params</tt> is <tt>null</tt>, 
     *                <tt>message</tt> is not to considered to be a MessageFormat-string.
     */
    public NotificationList (String message, Locale l, Object[] params) {
        super (message, l, params);
    }
    
    /**
     * Sets the type which may be one of the <tt>T_</tt>-constants. If this method
     * is not called, the type is defined by the contained <tt>Notification</tt>s with
     * the highest type-severity. 
     * 
     * @see INotification#getType()
     * @return this
     */
    public Notification setType (int type) {
        super.setType (type);
        autoType_ = false;
        return this;
    }
    
    /**
     * Returns the type. Either it has been explicitely set, then the one that
     * has been set is returned. Otherwise, the most severe type of the childs
     * is returned. 
     * 
     * @return one of the constants with praefix <tt>T_</tt> in {@link INotification}.
     * @see at.spardat.enterprise.exc.INotification#getType()
     */
    public int getType () {
        if (autoType_) {
            /**
             * the type is calculated from the types of the childs. 
             */
            int             maxSevere = 0;
            Iterator        iter = iterator();
            while (iter.hasNext()) {
                INotification noti = (INotification) iter.next();
                maxSevere = Math.max(maxSevere, noti.getType());
            }
            if (maxSevere == 0) {
                // the NotificationList does not contain a single Notification in the
                // complete subtree. In this case, the type defaults to T_ERROR (which
                // is the default type for INotifications).
                maxSevere = T_ERROR;
            }
            return maxSevere;
        } else {
            return super.getType();
        }
    }
    
    /**
     * Returns the number of Notifications contained in this that are
     * direct childs of this.
     */
    public int getChildCount () {
        return notifications_.size();
    }
    
    /**
     * Returns the number of Notifications contained in the subtree rooted at
     * this that are not <tt>NotificationList</tt>s.
     */
    public int getNotificationCount () {
        int             result = 0;
        Iterator iterator = iterator();
        while (iterator.hasNext()) {
            INotification noti = (INotification) iterator.next();
            if (noti instanceof NotificationList) result += ((NotificationList)noti).getNotificationCount();
            else result++;
        }
        return result;
    }
    
    /**
     * Returns an Iterator to iterate over the contained Notifications.
     */
    public Iterator iterator() {
        return notifications_.iterator();
    }
    
    /**
     * Adds a provided notification to the end of the list of notifications. Warning:
     * If the provided <tt>notification</tt> is a <tt>NotificationList</tt> itself, 
     * the list gets tree structured.
     * 
     * @param INotification the notification to add
     */
    public void add (INotification notification) {
        notifications_.add(notification);
    }
}
